// $Id: ConnectionParser.cpp,v 1.12 2014/02/10 20:50:48 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConnectionParser.h"
#include "XmlDef.h"
#include "XmlDocument.h"

#include <QFile>

//_________________________________________________________
ConnectionAttributes::List ConnectionParser::read( void )
{
    Debug::Throw( "ConnectionParser::read.\n" );
    ConnectionAttributes::List out;

    // clear read error
    error_.clear();

    // parse the file
    XmlDocument document;
    {
        QFile qfile( file_ );
        if( !document.setContent( &qfile ) )
        {
            error_.clear();
            QTextStream( &error_ )
                << QObject::tr( "An error occured while parsing document classes." ) << endl
                << document.error()
                << endl;
            return out;
        }
    }

    // look for relevant element
    QDomNodeList topNodes = document.elementsByTagName( Xml::ConnectionList );
    if( topNodes.isEmpty() ) return out;

    for(QDomNode node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
    {
        QDomElement element = node.toElement();
        if( element.isNull() ) continue;

        if( element.tagName() == Xml::Connection )
        { out << ConnectionAttributes( element ); }

    }

    return out;

}


//_________________________________________________________
void ConnectionParser::write( const ConnectionAttributes::List& attributes )
{
    Debug::Throw( "ConnectionParser::write.\n" );

    // create document and read
    XmlDocument document;
    {
        QFile qtfile( file_ );
        document.setContent( &qtfile );
    }

    // create main element
    QDomElement top = document.createElement( Xml::ConnectionList );
    for( ConnectionAttributes::List::const_iterator iter = attributes.begin(); iter != attributes.end(); ++iter )
    { top.appendChild( iter->domElement( document ) ); }

    document.replaceChild( top );
    {
        QFile qfile( file_ );
        if( !qfile.open( QIODevice::WriteOnly ) ) return;
        qfile.write( document.toByteArray() );
    }

    return;
}
