// $Id: MessageQueue.cpp,v 1.19 2014/02/10 10:34:30 hupereir Exp $


/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MessageQueue.h"
#include "MessageQueue.moc"

#include "Debug.h"
#include "MessageBox.h"
#include "Singleton.h"
#include "SvgEngine.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QDesktopWidget>


//______________________________________________________________
MessageQueue::MessageQueue( QObject* parent ):
    QObject( parent ),
    Counter( "MessageQueue" ),
    verticalOffset_( 0 ),
    messageBox_( 0 )
{
    Debug::Throw( "MessageQueue::MessageQueue.\n" );
    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();
}

//______________________________________________________________
MessageQueue::~MessageQueue( void )
{ Debug::Throw( "MessageQueue::~MessageQueue.\n" ); }


//______________________________________________________________
void MessageQueue::setVerticalOffset( int value )
{
    Debug::Throw() << "MessageQueue::setVerticalOffset - value: " << value << endl;
    verticalOffset_ = value;
}

//______________________________________________________________
void MessageQueue::processMessage( QString message )
{
    Debug::Throw() << "MessageQueue::processMessage - " << message << endl;
    if( !_hasMessageBox() ) _createMessageBox();
    _messageBox().addMessage( message );
    _messageBox().hideFromTaskBar();
    _updateGeometry();
    _messageBox().show();
}

//______________________________________________________________
void MessageQueue::_createMessageBox( void )
{

    // create dialog
    messageBox_ = new MessageBox( 0, "Notification from ssh tunnel daemon" );
    connect( &_messageBox(), SIGNAL(destroyed()), SLOT(_messageBoxDestroyed()) );

}

//______________________________________________________________
void MessageQueue::_updateGeometry( void )
{
    QSize size(
        qMax( _messageBox().sizeHint().width(), _messageBox().minimumSize().width() ),
        qMax( _messageBox().sizeHint().height(), _messageBox().minimumSize().height() ) );

    // get desktop size
    const QSize desktopSize( QApplication::desktop()->screenGeometry( QApplication::desktop()->screenNumber( &_messageBox() ) ).size() );

    int x( 0 );
    int y( 0 );
    if( _direction() == Descending )
    {
        y = _verticalOffset();
        x = desktopSize.width()-size.width();

    } else {

        y = _verticalOffset();
        y-= size.height();
        x = desktopSize.width()-size.width();

    }

    Debug::Throw() << "MessageQueue::processMessage - position: (" << x << "," << y << ")" << endl;
    _messageBox().setGeometry( QRect( QPoint( x,y ), size ) );

}

//____________________________________________________
void MessageQueue::_messageBoxDestroyed( void )
{
    Debug::Throw( "MessageQueue::_updateMessages.\n" );
    messageBox_ = 0;
}

//______________________________________________________________
void MessageQueue::_updateConfiguration( void )
{
    Debug::Throw( "MessageQueue::_updateConfiguration.\n" );

    // reload svg engine (needed for dialogs)
    Svg::SvgEngine::get().reload( true );

}
